# Default config for skidtool
# See documumention for details docs.clast.dev/skidtools/home/config/
# https://github.com/Abd0s/skidtools/

from typing import List, Dict, Union, Optional

# General
version: str = "0.0.1"
help_text: str = "A simple help accesed through --help"

# slogging
log_to_file: bool = False
debug: bool = False